//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  UsbFinder.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "UsbFinder.hpp"
#include "Error.hpp"
#include "Scanner.hpp"
#include "ModelInfo.hpp"
#include "Utility.hpp"
extern "C" {
#include <libusb-1.0/libusb.h>
}
#include <list>
#include <iostream>
namespace epsonscan {
static bool matchProductIDWithFlatbedOption(uint32_t productID)
{
  int j = 0;
  while(k2in1FlatbedOptionVID[j] != 0x0)
  {
   if(productID == k2in1FlatbedOptionVID[j])
   {
                return true;
    break;
   }
   j++;
  }
    return false;
}
void UsbFinder::Start(std::vector<SDIDeviceInfo>& devices, bool searchSecondDevice)
{
 try {
      libusb_context* ctx = nullptr;
  if(libusb_init(&ctx) == 0)
     {
        libusb_device **haystack;
        ssize_t cnt = libusb_get_device_list(ctx, &haystack);
        for (ssize_t i = 0; i < cnt; i++){
                struct libusb_device_descriptor dd;
            if (libusb_get_device_descriptor (haystack[i], &dd) == 0)
            {
                SDI_TRACE_LOG("USB device found VID = %d PID = %d", dd.idVendor, dd.idProduct);
                if (dd.idVendor == kEpsonUSBVID)
                {
                    std::string modelID;
                    if ( (searchSecondDevice && matchProductIDWithFlatbedOption(dd.idProduct) ) ||
                         ModelInfo::GetModelIDFromProductID(dd.idProduct, modelID))
                    {
                        SDIDeviceInfo devInfo = {0};
                        devInfo.productID = dd.idProduct;
                        bool isValidDeviceInfo = false;
                        strncpy(const_cast<char*>(devInfo.modelID), modelID.c_str(), MAX_MODEL_ID);
                        {
                            libusb_device_handle *devHandle = NULL;
                            if(libusb_open (haystack[i], &devHandle) == 0) {
                                unsigned char strDesc[256];
                                libusb_get_string_descriptor_ascii(devHandle, dd.iProduct, strDesc, 256);
                                 strncpy(const_cast<char*>(devInfo.displayName),(const char*)strDesc, MAX_DISPLAYNAME - 1);
                                 if (strlen(devInfo.displayName) != 0 || searchSecondDevice)
                                 {
                                     isValidDeviceInfo = true;
                                 }
                                libusb_close (devHandle);
                                devHandle = nullptr;
                            }
                        }
                        if (isValidDeviceInfo)
                        {
                            devices.push_back(devInfo);
                        }
                    }
                }
            }
        }
        libusb_free_device_list(haystack, 1);
     }else{
         SDI_TRACE_LOG("LibUSB Init fail");
     }
 libusb_exit(ctx);
 }catch(boost::exception& e) {
 }
}
}
