//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  PaperDeskew.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "PaperDeskew.hpp"
#include "Scanner.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void PaperDeskew::SetValue(const SDIInt& intVal)
{
 current_ = intVal;
}
void PaperDeskew::Reset()
{
 current_ = true;
}
void PaperDeskew::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
void PaperDeskew::GetCapability(SDICapability &capability)
{
 SetDefaultListCapability(capability);
 capability.supportLevel = kSDISupportLevelUnavailable;
 ESIndexSet available;
 Scanner* scanner = dataProvider_->GetScanner().get();
 if(scanner && scanner->GetAvailableValueForKey(kESSkewCorrection, available)) {
  if (available.find(true) != available.end())
     {
   AddList(capability, true);
   AddList(capability, false);
   AddAllList(capability, true);
   AddAllList(capability, false);
   capability.supportLevel = kSDISupportLevelAvailable;
  }else{
   AddList(capability, false);
   AddAllList(capability, false);
  }
 }
 if (GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitFlatbed)
 {
  capability.supportLevel = kSDISupportLevelUnavailable;
  ResetList(capability);
 }
}
ESDictionary PaperDeskew::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 dict[kESSkewCorrection] = (bool)current_;
 return dict;
}
}
