//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  ScanAreaHeight.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "ScanAreaHeight.hpp"
#include "Scanner.hpp"
#include "Resolution.hpp"
#include <ES2Command/ES2CommandAPI.h>
namespace epsonscan {
void ScanAreaHeight::SetValue(const SDIInt& intVal)
{
 current_ = intVal;
}
void ScanAreaHeight::Reset()
{
 current_ = 11.4 * 200;
}
void ScanAreaHeight::GetValue(SDIInt& intVal)
{
 intVal = current_;
}
void ScanAreaHeight::GetCapability(SDICapability &capability)
{
 SetDefaultRangeCapability(capability);
}
ESDictionary ScanAreaHeight::GetEngineKeyValues()
{
 auto dict = ESDictionary();
 SDIInt outputResolution = GetValueInt(kSDIResolutionKey);
 SDIInt scanResolutionX = dataProvider_->GetCommandValueAsNumber(kESXResolution);
 SDIInt scanResolutionY = dataProvider_->GetCommandValueAsNumber(kESYResolution);
 SDIInt offsetX = GetValueInt(kSDIScanAreaOffsetXKey) * scanResolutionX / outputResolution;
 SDIInt offsetY = GetValueInt(kSDIScanAreaOffsetYKey) * scanResolutionY / outputResolution;
 SDIInt width = GetValueInt(kSDIScanAreaWidthKey) * scanResolutionX / outputResolution;
 SDIInt height = GetValueInt(kSDIScanAreaHeightKey) * scanResolutionY / outputResolution;
 if (GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeStandard)
 {
  dict[kESScanSizeInPixel] = MakeSize<UInt32>(GetValueInt(kSDIMaxScanAreaWidthKey) / 100.0 * scanResolutionX,
             GetValueInt(kSDIMaxScanAreaHeightKey) / 100.0 * scanResolutionY);
  dict[kESPaperEndDetection] = (bool)false;
  return dict;
 }else if (GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeLongPaper)
 {
  dict[kESScanSizeInPixel] = MakeSize<UInt32>(GetValueInt(kSDIMaxScanAreaWidthKey) / 100.0 * scanResolutionX,
             std::min(120* scanResolutionY, GetValueInt(kSDIMaxScanAreaHeightInLongPaperKey)) / 100.0 * scanResolutionY);
        dict[kESPaperEndDetection] = (bool)true;
  dict[kESOverScan] = (bool)true;
  return dict;
 }else if (GetValueInt(kSDIPaperEndDetectionKey) == 1)
 {
  int maxHeight = std::min((int)(GetValueInt(kSDIMaxScanAreaHeightInLongPaperKey)/ 100.0 * scanResolutionY), 215 * 300);
  dict[kESScanSizeInPixel] = MakeSize<UInt32>(width, maxHeight);
        dict[kESPaperEndDetection] = (bool)true;
  return dict;
 }
 if (offsetX == 0 && offsetY== 0 )
 {
  dict[kESScanSizeInPixel] = MakeSize<UInt32>(width,
             height);
 }else{
  dict[kESScanAreaInPixel] = MakeRect<UInt32>(offsetX,
             offsetY,
             offsetX + width,
             offsetY + height);
 }
 dict[kESPaperEndDetection] = (bool)false;
 return dict;
}
}
