//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  backend.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "backend.h"
#include "version.hpp"
#include "assert.h"
#include "epsonscan.h"
#include "list.h"
#include "loadnetworksetting.h"
#include <sstream>
extern SANE_Status epsonscan_open (const char *, SANE_Handle *);
typedef struct
{
  void *cfg;
  void *net;
  list *sane_dev;
  void **dev_list;
} backend_type;
static backend_type *epson_backend = NULL;
static void be_sane_dev_dtor (void *);
static bool backend_is_usable = true;
static SANE_Auth_Callback auth_cb = NULL;
extern "C" {
SANE_Status check_error (SDIError inError);
SANE_Status
sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
{
  SANE_Status status = SANE_STATUS_GOOD;
  msg_init ();
  log_call("--------------sane_init--------------");
  if(epson_backend)
  {
      return status;
  }
  if (version_code)
    {
      *version_code = SANE_VERSION_CODE (BACKEND_MAJOR, BACKEND_MINOR,
                                             BACKEND_BUILD);
    }
  epson_backend = t_calloc (1, backend_type);
  if (!epson_backend)
    {
      return SANE_STATUS_NO_MEM;
    }
  auth_cb = authorize;
  return SANE_STATUS_GOOD;
}
void
sane_exit (void)
{
  log_call("--------------sane_exit--------------");
  if (!epson_backend)
    {
      return;
    }
  delete (epson_backend->dev_list);
  list_destroy (epson_backend->sane_dev, be_sane_dev_dtor);
  if(epson_backend){
    delete (epson_backend);
    epson_backend = NULL;
  }
  return;
}
#include <iostream>
SANE_Status
sane_get_devices (const SANE_Device ***device_list, SANE_Bool local_only)
{
  log_call("--------------sane_get_devices--------------");
  SANE_Status status = SANE_STATUS_GOOD;
  list *sane_dev = NULL;
  sane_dev = list_create ();
  Supervisor* sv = new Supervisor();
  sv->SetUp();
  if (sane_dev)
  {
    SDIInt count = 0;
    SDIDeviceFinder* finder = nullptr;
    sv->SDIDeviceFinder_CreatePtr_(&finder);
    sv->SDIDeviceFinder_StartDiscoveryPtr_(finder, nullptr, nullptr);
    sv->SDIDeviceFinder_StopDiscoveryPtr_(finder);
    SDIDeviceInfo* sdidevices = nullptr;
    sv->SDIDeviceFinder_GetDevicesPtr_(finder, &sdidevices, &count);
     std::list<SDIDeviceInfo> netdevs;
     GetNetworkDevcicesFromIni(netdevs);
    for (int i = 0; i < count; i++)
    {
      SANE_Device* dev = (SANE_Device*)malloc(sizeof(SANE_Device));
      if(!dev){
        status = SANE_STATUS_NO_MEM;
        goto DELETE;
      }
      {
        std::stringstream nameStream;
        nameStream << sdidevices[i].displayName;
        nameStream << ":esci2:usb:";
        nameStream << sdidevices[i].modelID;
        nameStream << ":";
        nameStream << sdidevices[i].productID;
        dev->name = (SANE_String)malloc(nameStream.str().length() + 1);
        memset((void*)dev->name, 0, nameStream.str().length() + 1);
        strcpy((char*)dev->name, nameStream.str().c_str() );
      }
      {
         std::string vender = "EPSON";
         dev->vendor = (SANE_String)malloc(vender.length() + 1);
         memset((void*)dev->vendor, 0, vender.length() + 1);
         strcpy((char*)dev->vendor, vender.c_str() );
      }
      {
         std::string model = sdidevices[i].displayName;;
         dev->model = (SANE_String)malloc(model.length() + 1);
         memset((void*)dev->model, 0, model.length() + 1);
         strcpy((char*)dev->model, model.c_str() );
      }
      {
         std::string type = "flatbed scanner";
         dev->type = (SANE_String)malloc(type.length() + 1);
         memset((void*)dev->type, 0, type.length() + 1);
         strcpy((char*)dev->type, type.c_str() );
      }
     list_append(sane_dev, dev);
    }
    for (auto netdev : netdevs)
    {
      SANE_Device* dev = (SANE_Device*)malloc(sizeof(SANE_Device));
      if(!dev){
        status = SANE_STATUS_NO_MEM;
        goto DELETE;
      }
      {
        std::stringstream nameStream;
        nameStream << "networkscanner:esci2:network:";
        nameStream << netdev.ipAddress;
        dev->name = (SANE_String)malloc(nameStream.str().length() + 1);
        memset((void*)dev->name, 0, nameStream.str().length() + 1);
        strcpy((char*)dev->name, nameStream.str().c_str() );
      }
      {
         std::string vender = "EPSON";
         dev->vendor = (SANE_String)malloc(vender.length() + 1);
         memset((void*)dev->vendor, 0, vender.length() + 1);
         strcpy((char*)dev->vendor, vender.c_str() );
      }
      {
         std::string model = "network scanner";;
         dev->model = (SANE_String)malloc(model.length() + 1);
         memset((void*)dev->model, 0, model.length() + 1);
         strcpy((char*)dev->model, model.c_str() );
      }
      {
         std::string type = "flatbed scanner";
         dev->type = (SANE_String)malloc(type.length() + 1);
         memset((void*)dev->type, 0, type.length() + 1);
         strcpy((char*)dev->type, type.c_str() );
      }
      list_append(sane_dev, dev);
    }
    if (epson_backend->sane_dev)
    {
      delete (epson_backend->dev_list);
      list_destroy (epson_backend->sane_dev, be_sane_dev_dtor);
    }
    epson_backend->sane_dev = sane_dev;
    sv->SDIDeviceFinder_DisposePtr_(finder);
    finder = nullptr;
  }
  epson_backend->dev_list = list_normalize (epson_backend->sane_dev);
  *device_list = (const SANE_Device **)epson_backend->dev_list;
  if (!*device_list)
  {
    status = SANE_STATUS_NO_MEM;
  }
  delete sv;
  sv = nullptr;
  return status;
DELETE:
  list_destroy (sane_dev, be_sane_dev_dtor);
  sane_dev = NULL;
  delete sv;
  sv = nullptr;
  return status;
}
SANE_Status
sane_open (SANE_String_Const device_name, SANE_Handle *handle)
{
  log_call("--------------sane_open--------------");
  log_call("%s", device_name);
  const SANE_Device *sane_dev = NULL;
  if (!epson_backend)
  {
    return SANE_STATUS_ACCESS_DENIED;
  }
  if (!handle)
  {
    return SANE_STATUS_INVAL;
  }
  if (!epson_backend->sane_dev)
    {
      const SANE_Device **dev = NULL;
      sane_get_devices (&dev, false);
    }
  if (0 == list_size (epson_backend->sane_dev))
    {
      return SANE_STATUS_ACCESS_DENIED;
    }
  if (!device_name || 0 == strlen (device_name))
    {
      sane_dev = (const SANE_Device *)epson_backend->sane_dev->head->data;
    }
  else
    {
      list_reset (epson_backend->sane_dev);
      while ((sane_dev = (const SANE_Device *)list_next (epson_backend->sane_dev))
             && 0 != strcmp (sane_dev->name, device_name))
        {
        }
    }
  if (!sane_dev)
  {
    return SANE_STATUS_INVAL;
  }
  return epsonscan_open(sane_dev->name, handle);
}
void
dip_flip_bits (SANE_Byte *buf, SANE_Int total_size)
{
  SANE_Byte *p;
  int i = 0;
  p = buf;
  while (i < total_size)
    {
      *p = ~*p;
      ++p;
      i++;
    }
}
SANE_Status
check_error (SDIError inError)
{
  SANE_Status status = SANE_STATUS_GOOD;
  log_call("Error Code : %d", inError);
  if(inError != kSDIErrorNone){
    if(inError == kSDIErrorPaperEmpty){
      log_call("ERROR : Load the originals in the ADF.");
      status = SANE_STATUS_NO_DOCS;
    }else if(inError == kSDIErrorPaperJam){
      log_call("ERROR : A paper jam has occurred. See the documentation for instructions on removing your originals.");
      status = SANE_STATUS_JAMMED;
    }else if(inError == kSDIErrorPaperDoubleFeed){
      log_call("ERROR : Double feed detected. See the documentation for instructions on removing your originals.");
      status = SANE_STATUS_JAMMED;
    }else if(inError == kSDIErrorPaperDoubleFeed2){
      log_call("ERROR : Double feed detected.");
      status = SANE_STATUS_JAMMED;
    }else if(inError == kSDIErrorPaperProtect){
      log_call("ERROR : A paper jam has occurred. See the documentation for instructions on removing your originals.");
      status = SANE_STATUS_JAMMED;
    }else if(inError == kSDIErrorDeviceInBusy){
      log_call("ERROR : kSDIErrorDeviceInBusy");
      status = SANE_STATUS_DEVICE_BUSY;
    }else if(inError == kSDIErrorCoverOpen){
      log_call("ERROR : ADF or ADF cover is open. Close it and reload the originals.");
      status = SANE_STATUS_COVER_OPEN;
    }else if(inError == kSDIErrorTrayClose){
      log_call("ERROR : The input tray is closed. Open the input tray.");
      status = SANE_STATUS_INVAL;
    }else if(inError == kSDIErrorDataSend){
      log_call("ERROR : Unable to send data. Check the connection to the scanner and try again.");
      status = SANE_STATUS_IO_ERROR;
    }else if(inError == kSDIErrorDataReceive){
      log_call("ERROR : Unable to receive data. Check the connection to the scanner and try again.");
      status = SANE_STATUS_IO_ERROR;
    }else if(inError == kSDIErrorDeviceInUse){
      log_call("ERROR : The scanner is in use or unavailable. Please wait.");
      status = SANE_STATUS_DEVICE_BUSY;
    }else if(inError == kSDIErrorNoMemory){
      log_call("ERROR : Not enough memory for Epson Scan 2. Close other applications and try again.");
      status = SANE_STATUS_NO_MEM;
    }else if(inError == kSDIErrorUnknownError){
      log_call("ERROR : An unexpected error occurred. Epson Scan 2 will close.");
      status = SANE_STATUS_INVAL;
    }else if(inError == kSDIErrorDeviceFormChangedInterruptedly){
      log_call("ERROR : DeviceFormChangedInterruptedly");
      status = SANE_STATUS_INVAL;
    }else if(inError == kSDIErrorDeviceFormUnstable){
      log_call("ERROR : DeviceFormUnstable");
      status = SANE_STATUS_INVAL;
    }else if(inError == kSDIErrorSepLeverChangedInterruptedly){
      log_call("ERROR : SepLeverChangedInterruptedly");
      status = SANE_STATUS_INVAL;
    }else if(inError == kSDIErrorUnscannableDeviceConfig1){
      log_call("ERROR : UnscannableDeviceConfig");
      status = SANE_STATUS_INVAL;
    }else if(inError == kSDIErrorETSensorError){
      log_call("ERROR : ETSensor");
      status = SANE_STATUS_INVAL;
    }else {
      log_call("ERROR : An unexpected error occurred. Epson Scan 2 will close.");
      status = SANE_STATUS_INVAL;
    }
  }
  return status;
}
SANE_Status
sane_get_parameters (SANE_Handle handle, SANE_Parameters *parameters)
{
  log_call("--------------sane_get_parameters--------------");
  Epson_Scanner *s = (Epson_Scanner *) handle;
  SANE_Status status = SANE_STATUS_GOOD;
  int width = 0;
  int height = 0;
  SDITransferEventType outType = kSDITrasnferEventTypeImage;
  unsigned int nSize = 100;
  unsigned char* tmp_buf = (unsigned char*)::malloc(nSize);
  SDIChar* output_path;
  if(tmp_buf){
    memset(tmp_buf, 0, nSize);
  }else {
    log_call("Status is SANE_STATUS_NO_MEM.");
    status = SANE_STATUS_NO_MEM;
    goto ERROR_2;
  }
  output_path = (SDIChar*)tmp_buf;
  SDIError outError;
  s->hw->sv->SDIImage_CreatePtr_(&s->outImageData);
  s->hw->sv->SDIScannerDriver_GetNextTransferEventPtr_(s->hw->sv->driver,
                                           &outType,
                                           s->outImageData,
                                           &outError);
  status = check_error(outError);
  if(status != SANE_STATUS_GOOD){
    log_call("Status is no SANE_STATUS_GOOD. %d", outError);
    goto ERROR;
  }
  if (outType == kSDITrasnferEventTypeImage)
  {
    log_call("kSDITrasnferEventTypeImage -> SDIImage_GetPathPtr_");
    s->hw->sv->SDIImage_GetPathPtr_(s->outImageData, output_path, 0);
  }else if (outType == kSDITransferEventTypeCancel){
    log_call("Button Cancel");
    s->cancel_requested = true;
    s->scan_complete = true;
  }else {
    log_call("Other EventType");
  }
  if(s->hw->sv->device_data.ColorType.select == kSDIColorTypeRGB24)
  {
    parameters->format = SANE_FRAME_RGB;
  }else if(s->hw->sv->device_data.ColorType.select == kSDIColorTypeMono8 ||
          s->hw->sv->device_data.ColorType.select == kSDIColorTypeMono1)
  {
    parameters->format = SANE_FRAME_GRAY;
  }
  width = s->hw->sv->SDIImage_GetWidthPtr_(s->outImageData);
  height = s->hw->sv->SDIImage_GetHeightPtr_(s->outImageData);
  parameters->lines = height;
  parameters->depth = s->hw->sv->SDIImage_GetBitsPerSamplePtr_(s->outImageData);
  s->hw->ChannelPerPixel = s->hw->sv->SDIImage_GetSamplesPerPixelPtr_(s->outImageData);
  parameters->bytes_per_line = (width * s->hw->ChannelPerPixel * parameters->depth + 7) / 8;
  parameters->pixels_per_line = width;
  s->total_datasize = parameters->bytes_per_line * height;
  log_call("width : %d", width);
  log_call("height : %d", height);
  log_call("parameters->last_frame : %d", parameters->last_frame);
  log_call("parameters->format : %d", parameters->format);
  log_call("parameters->bytes_per_line : %d", parameters->bytes_per_line);
  log_call("parameters->pixels_per_line : %d", parameters->pixels_per_line);
  log_call("parameters->lines : %d", parameters->lines);
  log_call("parameters->depth : %d", parameters->depth);
  log_call("total_datasize : %d", s->total_datasize);
  if (s->cancel_requested == true){
    log_call("Cancel requested");
    goto ERROR;
  }
  s->cur_pos = 0;
  if(output_path){
    FILE *fp = nullptr;
    log_call("output_path : %s", output_path);
    s->out_buffer = (SANE_Byte *)malloc(s->total_datasize);
    if(!s->out_buffer){
      status = SANE_STATUS_NO_MEM;
      goto ERROR;
    }
    if((fp = fopen(output_path, "rb")) != nullptr){
      if(fread(s->out_buffer, 1, s->total_datasize, fp) != s->total_datasize){
        status = SANE_STATUS_INVAL;
        if(fp != nullptr){
          fclose(fp);
        }
        goto ERROR;
      }else {
        if(fp != nullptr){
          fclose(fp);
        }
        parameters->last_frame = SANE_TRUE;
      }
    }else {
      status = SANE_STATUS_INVAL;
      fclose(fp);
      goto ERROR;
    }
    if(s->hw->sv->device_data.ColorType.select == kSDIColorTypeMono1){
      dip_flip_bits(s->out_buffer, s->total_datasize);
    }
  }
ERROR:
  s->hw->sv->SDIImage_DisposePtr_(s->outImageData);
ERROR_2:
  if(tmp_buf){
    free(tmp_buf);
    tmp_buf = nullptr;
  }
  if(s->cancel_requested == true){
    log_call("Cancel requested2");
    status = SANE_STATUS_CANCELLED;
  }
  return status;
}
SANE_Status
sane_read (SANE_Handle handle, SANE_Byte *buffer, SANE_Int max_length,
           SANE_Int *length)
{
  log_call("--------------sane_read--------------");
  SANE_Status status = SANE_STATUS_GOOD;
  Epson_Scanner *s;
  if (length) *length = 0;
  if (!handle)
    {
      return SANE_STATUS_INVAL;
    }
  s = (Epson_Scanner *) handle;
  if(s->total_datasize == 0){
    log_call("image data -> EOF");
    s->image_count++;
    s->scan_complete = true;
    return SANE_STATUS_EOF;
  }else if(length){
    if((s->total_datasize - max_length) < 0){
      log_call("Last data size : %d", s->total_datasize);
      *length = s->total_datasize;
    }else {
      *length = max_length;
    }
    memcpy(buffer, s->out_buffer + s->cur_pos, *length);
    s->total_datasize -= *length;
    s->cur_pos += *length;
    log_call("length : %d", *length);
  }
  log_call("cur_pos : %d", s->cur_pos);
  return status;
}
void
sane_cancel (SANE_Handle handle)
{
  log_call("--------------sane_cancel--------------");
  SANE_Status status = SANE_STATUS_GOOD;
  Epson_Scanner *s;
  s = (Epson_Scanner *) handle;
  s->image_count = 0;
  if(s->scan_complete == false){
    if(s->cancel_requested == false){
      s->cancel_requested = true;
    }else if(s->cancel_requested == true){
      log_call("Scan cancel terminated.");
      s->hw->sv->SDIScannerDriver_DoScanJobPtr_(s->hw->sv->driver, kSDIOperationTypeCancel);
    }
  }
  log_call("--------------sane_cancel--------------");
  return;
}
SANE_Status
sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
{
  log_call("--------------sane_set_io_mode--------------");
 handle = handle;
 non_blocking = non_blocking;
 return SANE_STATUS_UNSUPPORTED;
}
SANE_Status
sane_get_select_fd (SANE_Handle handle, SANE_Int *fdp)
{
  log_call("--------------sane_get_select_fd--------------");
 handle = handle;
 return SANE_STATUS_UNSUPPORTED;
}
}
static void
be_sane_dev_dtor (void *p)
{
  SANE_Device *sd = (SANE_Device *) p;
  if (!sd) return;
  free((void*)sd->name);
  free((void*)sd->vendor);
  free((void*)sd->type);
  free((void*)sd->model);
  delete (sd);
}
