//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  epson.h -- template and derived ESC/I ESC/I-2 protocol commands

#ifndef epson_h
#define epson_h 1
#define SANE_EPSON_CONFIG_USB "usb"
#define SANE_EPSON_CONFIG_PIO "pio"
#define SANE_NAME_GAMMA_CORRECTION "gamma-correction"
#define SANE_TITLE_GAMMA_CORRECTION "Gamma Correction"
#define SANE_DESC_GAMMA_CORRECTION "Selectes the gamma correction value from a list of pre-defined devices or the user defined table, which can be downloaded to the scanner"
#define LINES_SHUFFLE_MAX (17)
typedef struct {
 unsigned char * level;
 unsigned char request_identity;
 unsigned char request_identity2;
 unsigned char request_status;
 unsigned char request_condition;
 unsigned char set_color_mode;
 unsigned char start_scanning;
 unsigned char set_data_format;
 unsigned char set_resolution;
 unsigned char set_zoom;
 unsigned char set_scan_area;
 unsigned char set_bright;
 SANE_Range bright_range;
 unsigned char set_gamma;
 unsigned char set_halftoning;
 unsigned char set_color_correction;
 unsigned char initialize_scanner;
 unsigned char set_speed;
 unsigned char set_lcount;
 unsigned char mirror_image;
 unsigned char set_gamma_table;
 unsigned char set_outline_emphasis;
 unsigned char set_dither;
 unsigned char set_color_correction_coefficients;
 unsigned char request_extension_status;
 unsigned char control_an_extension;
 unsigned char eject;
 unsigned char request_push_button_status;
 unsigned char control_auto_area_segmentation;
 unsigned char set_film_type;
 unsigned char set_exposure_time;
 unsigned char set_bay;
 unsigned char set_threshold;
} EpsonCmdRec, * EpsonCmd;
enum
 { OPT_NUM_OPTS = 0
 , OPT_MODE_GROUP
  , OPT_MODE
  , OPT_HALFTONE
  , OPT_DROPOUT
  , OPT_BRIGHTNESS
  , OPT_SHARPNESS
  , OPT_GAMMA_CORRECTION
  , OPT_COLOR_CORRECTION
  , OPT_RESOLUTION
  , OPT_THRESHOLD
 , OPT_ADVANCED_GROUP
  , OPT_MIRROR
  , OPT_SPEED
  , OPT_AAS
  , OPT_ZOOM
  , OPT_GAMMA_VECTOR
  , OPT_GAMMA_VECTOR_R
  , OPT_GAMMA_VECTOR_G
  , OPT_GAMMA_VECTOR_B
 , OPT_CCT_GROUP
  , OPT_CCT_1
  , OPT_CCT_2
  , OPT_CCT_3
  , OPT_CCT_4
  , OPT_CCT_5
  , OPT_CCT_6
  , OPT_CCT_7
  , OPT_CCT_8
  , OPT_CCT_9
 , OPT_PREVIEW_GROUP
  , OPT_PREVIEW
  , OPT_PREVIEW_SPEED
 , OPT_GEOMETRY_GROUP
  , OPT_TL_X
  , OPT_TL_Y
  , OPT_BR_X
  , OPT_BR_Y
  , OPT_QUICK_FORMAT
 , OPT_EQU_GROUP
  , OPT_SOURCE
  , OPT_AUTO_EJECT
  , OPT_FILM_TYPE
  , OPT_BAY
  , OPT_EJECT
 , NUM_OPTIONS
 };
typedef enum {
        SANE_EPSON_NODEV,
 SANE_EPSON_SCSI,
 SANE_EPSON_PIO,
 SANE_EPSON_USB
} Epson_Connection_Type;
typedef struct
{
 u_short opt_resolution;
 u_char sensor;
 u_char scan_order;
 u_char line_dist1;
 u_char line_dist2;
 u_short main_res1;
 u_short main_res2;
 u_short main_res3;
 u_short main_res4;
 u_short main_res5;
 u_short main_res6;
 u_short main_res7;
 u_short sub_res1;
 u_short sub_res2;
 u_short sub_res3;
 u_short sub_res4;
 u_short sub_res5;
 u_short sub_res6;
} Epson_Identity2;
struct Epson_Device {
        struct Epson_Device *next;
 SANE_Device sane;
 SANE_Int level;
 SANE_Range dpi_range;
 SANE_Range * x_range;
 SANE_Range * y_range;
 SANE_Range fbf_x_range;
 SANE_Range fbf_y_range;
 SANE_Range adf_x_range;
 SANE_Range adf_y_range;
 SANE_Range tpu_x_range;
 SANE_Range tpu_y_range;
 Epson_Connection_Type connection;
 SANE_Int *res_list;
 SANE_Int res_list_size;
 SANE_Int last_res;
 SANE_Int last_res_preview;
 SANE_Word *resolution_list;
 SANE_Bool extension;
 SANE_Bool use_extension;
 SANE_Bool TPU;
 SANE_Bool ADF;
 SANE_Bool color_shuffle;
 SANE_Int optical_res;
 SANE_Int max_line_distance;
 EpsonCmd cmd;
};
typedef struct Epson_Device Epson_Device;
typedef union {
 SANE_Word w;
 SANE_Word * wa;
 SANE_String s;
} Option_Value;
struct Epson_Scanner {
 struct Epson_Scanner *next;
 int fd;
 Epson_Device * hw;
 SANE_Option_Descriptor opt [ NUM_OPTIONS];
 Option_Value val [ NUM_OPTIONS];
 SANE_Parameters params;
 SANE_Bool block;
 SANE_Bool eof;
 SANE_Byte * buf, * end, * ptr;
 SANE_Bool canceling;
 SANE_Bool invert_image;
 SANE_Word gamma_table [ 4] [ 256];
 SANE_Int retry_count;
 SANE_Byte *line_buffer[LINES_SHUFFLE_MAX];
 SANE_Int color_shuffle_line;
 SANE_Int line_distance;
 SANE_Int current_output_line;
 SANE_Int lines_written;
};
typedef struct Epson_Scanner Epson_Scanner;
#endif
