//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  handle.hpp -- template and derived ESC/I ESC/I-2 protocol commands

#ifndef sane_handle_hpp_
#define sane_handle_hpp_ 
extern "C" {
#include <sane/sane.h>
}
#include <csignal>
#include <string>
#include <vector>
#include <boost/operators.hpp>
namespace sane {
class handle
{
public:
  handle (const utsushi::scanner::info& info);
  std::string name () const;
  SANE_Int size () const;
  const SANE_Option_Descriptor * descriptor (SANE_Int index) const;
  bool is_active (SANE_Int index) const;
  bool is_button (SANE_Int index) const;
  bool is_group (SANE_Int index) const;
  bool is_settable (SANE_Int index) const;
  bool is_automatic (SANE_Int index) const;
  bool is_scanning () const;
  SANE_Status get (SANE_Int index, void *value) const;
  SANE_Status set (SANE_Int index, void *value, SANE_Word *info);
  SANE_Status set (SANE_Int index, SANE_Word *info);
  utsushi::context get_context () const;
  utsushi::streamsize start ();
  utsushi::streamsize read (utsushi::octet *buffer,
                            utsushi::streamsize length);
  void cancel ();
protected:
  void end_scan_sequence ();
  utsushi::streamsize marker ();
  std::string name_;
  utsushi::scanner::ptr idev_;
  utsushi::idevice::ptr cache_;
  utsushi::pump::ptr pump_;
  utsushi::weak_ptr< utsushi::idevice > iptr_;
  utsushi::streamsize last_marker_;
  sig_atomic_t work_in_progress_;
  sig_atomic_t cancel_requested_;
private:
  void add_option (utsushi::option& visitor);
  void update_options (SANE_Word *info);
  void update_capabilities (SANE_Word *info);
  utsushi::option::map opt_;
  struct option_descriptor
    : SANE_Option_Descriptor
    , private boost::equality_comparable< option_descriptor >
  {
    utsushi::key orig_key;
    std::string sane_key;
    utsushi::string name_;
    utsushi::string desc_;
    std::vector< utsushi::string > strings_;
    option_descriptor ();
    option_descriptor (const option_descriptor& od);
    option_descriptor (const utsushi::option& visitor);
    ~option_descriptor ();
    option_descriptor& operator= (const option_descriptor& rhs);
    bool operator== (const option_descriptor& rhs) const;
  };
  void add_group (const utsushi::key& key,
                  const utsushi::string& name,
                  const utsushi::string& text = utsushi::string ());
  std::vector< option_descriptor > sod_;
  bool emulating_automatic_scan_area_;
  bool do_automatic_scan_area_;
  friend struct match_key;
};
}
#endif
